@@ -1,5 +1,4 @@
 ''' Main simulation-side stack functions. '''
-from fnmatch import fnmatch
 import math
 from pathlib import Path
 import traceback
@@ -138,11 +137,13 @@ def process(from_pcall=None):
 
 def readscn(fname):
     ''' Read a scenario file. '''
+    if not fname:
+        return
     # Ensure .scn suffix and specify path if necessary
     fname = Path(fname).with_suffix('.scn')
     if not fname.is_absolute():
-        fname = Path(settings.scenario_path) / fname
-    
+        fname = bs.resource(settings.scenario_path) / fname
+
     with open(fname, "r") as fscen:
         prevline = ''
         for line in fscen:
@@ -264,31 +265,33 @@ def ic(filename : 'string' = ''):
     # Get the filename of new scenario
     if not filename:
         filename = bs.scr.show_file_dialog()
+        if not filename:
+            # Only PyGame returns a filename from the dialog here
+            return
 
     # Clean up filename
     filename = Path(filename)
 
     # Reset sim and open new scenario file
-    if filename:
-        try:
-            for (cmdtime, cmd) in readscn(filename):
-                Stack.scentime.append(cmdtime)
-                Stack.scencmd.append(cmd)
-            Stack.scenname = filename.stem
+    try:
+        for (cmdtime, cmd) in readscn(filename):
+            Stack.scentime.append(cmdtime)
+            Stack.scencmd.append(cmd)
+        Stack.scenname = filename.stem
 
-            # Remember this filename in IC.scn in scenario folder
-            with open(Path(settings.scenario_path) / "ic.scn", "w") as keepicfile:
-                keepicfile.write(
-                    "# This file is used by BlueSky to save the last used scenario file\n"
-                )
-                keepicfile.write(
-                    "# So in the console type 'IC IC' to restart the previously used scenario file\n"
-                )
-                keepicfile.write(f"00:00:00.00>IC {filename}\n")
+        # Remember this filename in IC.scn in scenario folder
+        with open(bs.resource(settings.scenario_path) / "ic.scn", "w") as keepicfile:
+            keepicfile.write(
+                "# This file is used by BlueSky to save the last used scenario file\n"
+            )
+            keepicfile.write(
+                "# So in the console type 'IC IC' to restart the previously used scenario file\n"
+            )
+            keepicfile.write(f"00:00:00.00>IC {filename}\n")
 
-            return True, f"IC: Opened {filename}"
-        except FileNotFoundError:
-            return False, f"IC: File not found: {filename}"
+        return True, f"IC: Opened {filename}"
+    except FileNotFoundError:
+        return False, f"IC: File not found: {filename}"
 
 
 @command(aliases=('SCEN',))
@@ -388,7 +391,7 @@ def makedoc():
     # Get unique set of commands
     cmdobjs = set(Command.cmddict.values())
     for o in cmdobjs:
-        if not Path(f"data/html/{o.name}.html").is_file():
+        if not bs.resource(f"html/{o.name}.html").is_file():
             with open(tmp / f"{o.name.lower()}.md", "w") as f:
                 f.write(
                     f"# {o.name}: {o.name.capitalize()}\n"
