@@ -126,9 +126,9 @@ class Traffic(Entity):
             self.selvs  = np.array([])  # selected vertical speed [m/s]
 
             # Whether to perform LNAV and VNAV
-            self.swlnav    = np.array([], dtype=np.bool)
-            self.swvnav    = np.array([], dtype=np.bool)
-            self.swvnavspd = np.array([], dtype=np.bool)
+            self.swlnav    = np.array([], dtype=bool)
+            self.swvnav    = np.array([], dtype=bool)
+            self.swvnavspd = np.array([], dtype=bool)
 
             # Flight Models
             self.cd       = ConflictDetection()
@@ -144,10 +144,10 @@ class Traffic(Entity):
             self.groups = TrafficGroups()
 
             # Traffic autopilot data
-            self.swhdgsel = np.array([], dtype=np.bool)  # determines whether aircraft is turning
+            self.swhdgsel = np.array([], dtype=bool)  # determines whether aircraft is turning
 
             # Traffic autothrottle settings
-            self.swats    = np.array([], dtype=np.bool)  # Switch indicating whether autothrottle system is on/off
+            self.swats    = np.array([], dtype=bool)  # Switch indicating whether autothrottle system is on/off
             self.thr      = np.array([])        # Thottle seeting (0.0-1.0), negative = non-valid/auto
 
             # Display information on label
@@ -301,6 +301,7 @@ class Traffic(Entity):
             for cmdtxt in self.crecmdlist:
                  bs.stack.stack(self.id[j]+" "+cmdtxt)
 
+        return True
 
     def creconfs(self, acid, actype, targetidx, dpsi, dcpa, tlosh, dH=None, tlosv=None, spd=None):
         ''' Create an aircraft in conflict with target aircraft.
